@@ -6,8 +6,7 @@
 import datetime
 import functools
 import logging
-
-from typing import Container, Dict, Optional
+from typing import Any, Container, Dict, Optional
 
 import pytest
 
@@ -40,7 +39,7 @@ def replayer_storage_and_client(
         "client_id": "kafka_writer",
         "prefix": kafka_prefix,
     }
-    storage_config = {
+    storage_config: Dict[str, Any] = {
         "cls": "memory",
         "journal_writer": journal_writer_config,
     }
@@ -201,6 +200,7 @@ def _check_replayed(
         "snapshots",
         "origins",
         "origin_visits",
+        "origin_visit_statuses",
     ):
         if exclude and attr in exclude:
             continue
@@ -277,16 +277,15 @@ def test_storage_play_anonymized(
         "prefix": kafka_prefix,
         "anonymize": True,
     }
-    src_config = {"cls": "memory", "journal_writer": writer_config}
+    src_config: Dict[str, Any] = {"cls": "memory", "journal_writer": writer_config}
 
     storage = get_storage(**src_config)
 
     # Fill the src storage
     nb_sent = 0
     for obj_type, objs in TEST_OBJECTS.items():
-        if obj_type == "origin_visit":
-            # these have non-consistent API and are unrelated with what we
-            # want to test here
+        if obj_type in ("origin_visit", "origin_visit_status"):
+            # these are unrelated with what we want to test here
             continue
         method = getattr(storage, obj_type + "_add")
         method(objs)
@@ -348,6 +347,7 @@ def check_replayed(src, dst, expected_anonymized=False):
         "releases",
         "snapshots",
         "origins",
+        "origin_visit_statuses",
     ):
         expected_objects = [
             (id, maybe_anonymize(obj))
